@@ -15,12 +15,17 @@
 /* 08.12.03  2.01  Dan  High-memory handling (> 0x80000000)                  */
 /* 09.01.13  3.00  Lad  Refactored, beautified, documented :-)               */
 /*****************************************************************************/
- 
+
 #include <assert.h>
 #include <string.h>
- 
+
 #include "huff.h"
 
+//-----------------------------------------------------------------------------
+// Local defined
+
+#define HUFF_DECOMPRESS_ERROR   0x1FF
+
 //-----------------------------------------------------------------------------
 // Table of byte-to-weight values
 
@@ -67,7 +72,7 @@ static unsigned char ByteToWeight_01[] =
     0x02, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x02, 0x02, 0x01, 0x01, 0x02, 0x02, 0x02, 0x06, 0x4B,
     0x00, 0x00
 };
-   
+
 // Data for compression type 0x02
 static unsigned char ByteToWeight_02[] =
 {
@@ -89,7 +94,7 @@ static unsigned char ByteToWeight_02[] =
     0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
     0x00, 0x00
 };
-   
+
 // Data for compression type 0x03
 static unsigned char ByteToWeight_03[] =
 {
@@ -111,7 +116,7 @@ static unsigned char ByteToWeight_03[] =
     0x02, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x02, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x11,
     0x00, 0x00
 };
-   
+
 // Data for compression type 0x04
 static unsigned char ByteToWeight_04[] =
 {
@@ -133,7 +138,7 @@ static unsigned char ByteToWeight_04[] =
     0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
     0x00, 0x00
 };
-   
+
 // Data for compression type 0x05
 static unsigned char ByteToWeight_05[] =
 {
@@ -155,7 +160,7 @@ static unsigned char ByteToWeight_05[] =
     0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
     0x00, 0x00
 };
-   
+
     // Data for compression type 0x06
 static unsigned char ByteToWeight_06[] =
 {
@@ -177,7 +182,7 @@ static unsigned char ByteToWeight_06[] =
     0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
     0x00, 0x00
 };
-   
+
 // Data for compression type 0x07
 static unsigned char ByteToWeight_07[] =
 {
@@ -199,7 +204,7 @@ static unsigned char ByteToWeight_07[] =
     0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
     0x00, 0x00
 };
-   
+
 // Data for compression type 0x08
 static unsigned char ByteToWeight_08[] =
 {
@@ -235,7 +240,7 @@ static unsigned char * WeightTables[0x09] =
     ByteToWeight_08
 };
 
-//----------------------------------------------------------------------------- 
+//-----------------------------------------------------------------------------
 // Debug/diagnostics
 
 #ifdef _DEBUG
@@ -269,65 +274,75 @@ TInputStream::TInputStream(void * pvInBuffer, size_t cbInBuffer)
     BitCount = 0;
 }
 
-// Gets 7 bits from the stream. DOES NOT remove the bits from input stream
-unsigned int TInputStream::Peek7Bits()
-{
-    unsigned int dwReloadByte = 0;
-
-    // If there is not enough bits to get the value,
-    // we have to add 8 more bits from the input buffer
-    if(BitCount < 7)
-    {
-        dwReloadByte = *pbInBuffer++;
-        BitBuffer |= dwReloadByte << BitCount;
-        BitCount += 8;
-    }
-
-    // Return the first available 7 bits. DO NOT remove them from the input stream
-    return (BitBuffer & 0x7F);
-}
-
 // Gets one bit from input stream
-unsigned int TInputStream::Get1Bit()
+bool TInputStream::Get1Bit(unsigned int & BitValue)
 {
-    unsigned int OneBit = 0;
-
     // Ensure that the input stream is reloaded, if there are no bits left
     if(BitCount == 0)
     {
+        // Buffer overflow check
+        if(pbInBuffer >= pbInBufferEnd)
+            return false;
+
         // Refill the bit buffer
         BitBuffer = *pbInBuffer++;
         BitCount = 8;
     }
 
     // Copy the bit from bit buffer to the variable
-    OneBit = (BitBuffer & 0x01);
+    BitValue = (BitBuffer & 0x01);
     BitBuffer >>= 1;
     BitCount--;
-
-    return OneBit;
-}   
+    return true;
+}
 
 // Gets the whole byte from the input stream.
-unsigned int TInputStream::Get8Bits()
+bool TInputStream::Get8Bits(unsigned int & ByteValue)
 {
     unsigned int dwReloadByte = 0;
-    unsigned int dwOneByte = 0;
 
     // If there is not enough bits to get the value,
     // we have to add 8 more bits from the input buffer
     if(BitCount < 8)
     {
+        // Buffer overflow check
+        if(pbInBuffer >= pbInBufferEnd)
+            return false;
+
         dwReloadByte = *pbInBuffer++;
         BitBuffer |= dwReloadByte << BitCount;
         BitCount += 8;
     }
 
     // Return the lowest 8 its
-    dwOneByte = (BitBuffer & 0xFF);
+    ByteValue = (BitBuffer & 0xFF);
     BitBuffer >>= 8;
     BitCount -= 8;
-    return dwOneByte;
+    return true;
+}
+
+// Gets 7 bits from the stream. DOES NOT remove the bits from input stream
+bool TInputStream::Peek7Bits(unsigned int & Value)
+{
+    unsigned int Value8Bits = 0;
+
+    // If there is not enough bits to get the value,
+    // we have to add 8 more bits from the input buffer
+    if(BitCount < 7)
+    {
+        // Load additional 8 bits. Be careful if we have no more data
+        if(pbInBuffer >= pbInBufferEnd)
+            return false;
+        Value8Bits = *pbInBuffer++;
+
+        // Add these 8 bits to the bit buffer
+        BitBuffer |= Value8Bits << BitCount;
+        BitCount += 8;
+    }
+
+    // Return 7 bits of data. DO NOT remove them from the input stream
+    Value = (BitBuffer & 0x7F);
+    return true;
 }
 
 void TInputStream::SkipBits(unsigned int dwBitsToSkip)
@@ -338,6 +353,10 @@ void TInputStream::SkipBits(unsigned int dwBitsToSkip)
     // we have to add 8 more bits from the input buffer
     if(BitCount < dwBitsToSkip)
     {
+        // Buffer overflow check
+        if(pbInBuffer >= pbInBufferEnd)
+            return;
+
         dwReloadByte = *pbInBuffer++;
         BitBuffer |= dwReloadByte << BitCount;
         BitCount += 8;
@@ -363,13 +382,13 @@ void TOutputStream::PutBits(unsigned int dwValue, unsigned int nBitCount)
 {
     BitBuffer |= (dwValue << BitCount);
     BitCount += nBitCount;
- 
+
     // Flush completed bytes
     while(BitCount >= 8)
     {
         if(pbOutBuffer < pbOutBufferEnd)
             *pbOutBuffer++ = (unsigned char)BitBuffer;
- 
+
         BitBuffer >>= 8;
         BitCount -= 8;
     }
@@ -402,7 +421,7 @@ void THTreeItem::RemoveItem()
 
 //-----------------------------------------------------------------------------
 // THuffmannTree class functions
- 
+
 THuffmannTree::THuffmannTree(bool bCompression)
 {
     pFirst = pLast = LIST_HEAD();
@@ -439,7 +458,7 @@ void THuffmannTree::InsertItem(THTreeItem * pNewItem, TInsertPoint InsertPoint,
 {
     // Remove the item from the tree
     pNewItem->RemoveItem();
- 
+
     if(pInsertPoint == NULL)
         pInsertPoint = LIST_HEAD();
 
@@ -448,7 +467,7 @@ void THuffmannTree::InsertItem(THTreeItem * pNewItem, TInsertPoint InsertPoint,
         case InsertAfter:
             LinkTwoItems(pInsertPoint, pNewItem);
             return;
-       
+
         case InsertBefore:
             pNewItem->pNext = pInsertPoint;             // Set next item (or pointer to pointer to first item)
             pNewItem->pPrev = pInsertPoint->pPrev;      // Set prev item (or last item in the tree)
@@ -495,7 +514,7 @@ THTreeItem * THuffmannTree::CreateNewItem(unsigned int DecompressedValue, unsign
         pNewItem->pParent = NULL;
         pNewItem->pChildLo = NULL;
     }
-    
+
     return pNewItem;
 }
 
@@ -529,16 +548,16 @@ bool THuffmannTree::BuildTree(unsigned int CompressionType)
     THTreeItem * pChildHi;
     unsigned char * WeightTable;
     unsigned int MaxWeight;                     // [ESP+10] - The greatest character found in table
- 
+
     // Clear all pointers in HTree item array
     memset(ItemsByByte, 0, sizeof(ItemsByByte));
     MaxWeight = 0;
- 
+
     // Ensure that the compression type is in range
     if((CompressionType & 0x0F) > 0x08)
         return false;
     WeightTable  = WeightTables[CompressionType & 0x0F];
- 
+
     // Build the linear list of entries that is sorted by byte weight
     for(unsigned int i = 0; i < 0x100; i++)
     {
@@ -552,11 +571,11 @@ bool THuffmannTree::BuildTree(unsigned int CompressionType)
             MaxWeight = FixupItemPosByWeight(pNewItem, MaxWeight);
         }
     }
- 
+
     // Insert termination entries at the end of the list
     ItemsByByte[0x100] = CreateNewItem(0x100, 1, InsertBefore);
     ItemsByByte[0x101] = CreateNewItem(0x101, 1, InsertBefore);
- 
+
     // Now we need to build the tree. We start at the last entry
     // and go backwards to the first one
     pChildLo = pLast;
@@ -598,7 +617,7 @@ void THuffmannTree::IncWeightsAndRebalance(THTreeItem * pItem)
     THTreeItem * pChildHi;              // The higher-weight child
     THTreeItem * pChildLo;              // The lower-weight child
     THTreeItem * pParent;
- 
+
     // Climb up the tree and increment weight of each tree item
     for(; pItem != NULL; pItem = pItem->pParent)
     {
@@ -615,11 +634,11 @@ void THuffmannTree::IncWeightsAndRebalance(THTreeItem * pItem)
             // Move the previous item to the RIGHT from the given item
             pChildHi->RemoveItem();
             LinkTwoItems(pItem, pChildHi);
-            
+
             // Move the given item AFTER the greater-weight tree item
             pItem->RemoveItem();
             LinkTwoItems(pHigherItem, pItem);
-     
+
             // We need to maintain the tree so that pChildHi->Weight is >= pChildLo->Weight.
             // Rebalance the tree accordingly.
             pChildLo = pChildHi->pParent->pChildLo;
@@ -696,16 +715,13 @@ unsigned int THuffmannTree::DecodeOneByte(TInputStream * is)
     THTreeItem * pItem;
     unsigned int ItemLinkIndex;
     unsigned int BitCount = 0;
+    bool bHasItemLinkIndex;
 
-    // Check for the end of the input stream
-    if(is->pbInBuffer >= is->pbInBufferEnd && is->BitCount < 7)
-        return 0x1FF;
+    // Try to retrieve quick link index
+    bHasItemLinkIndex = is->Peek7Bits(ItemLinkIndex);
 
-    // Get the eventual quick-link index
-    ItemLinkIndex = is->Peek7Bits();
-    
     // Is the quick-link item valid?
-    if(QuickLinks[ItemLinkIndex].ValidValue > MinValidValue)
+    if(bHasItemLinkIndex && QuickLinks[ItemLinkIndex].ValidValue > MinValidValue)
     {
         // If that item needs less than 7 bits, we can get decompressed value directly
         if(QuickLinks[ItemLinkIndex].ValidBits <= 7)
@@ -723,7 +739,7 @@ unsigned int THuffmannTree::DecodeOneByte(TInputStream * is)
     {
         // Just a sanity check
         if(pFirst == LIST_HEAD())
-            return 0x1FF;
+            return HUFF_DECOMPRESS_ERROR;
 
         // We don't have the quick-link item, we need to parse the tree from its root
         pItem = pFirst;
@@ -732,9 +748,14 @@ unsigned int THuffmannTree::DecodeOneByte(TInputStream * is)
     // Step down the tree until we find a terminal item
     while(pItem->pChildLo != NULL)
     {
+        unsigned int BitValue = 0;
+
         // If the next bit in the compressed stream is set, we get the higher-weight
         // child. Otherwise, get the lower-weight child.
-        pItem = is->Get1Bit() ? pItem->pChildLo->pPrev : pItem->pChildLo;
+        if(!is->Get1Bit(BitValue))
+            return HUFF_DECOMPRESS_ERROR;
+
+        pItem = BitValue ? pItem->pChildLo->pPrev : pItem->pChildLo;
         BitCount++;
 
         // If the number of loaded bits reached 7,
@@ -745,7 +766,7 @@ unsigned int THuffmannTree::DecodeOneByte(TInputStream * is)
 
     // If we didn't get the item from the quick-link array,
     // set the entry in it
-    if(QuickLinks[ItemLinkIndex].ValidValue < MinValidValue)
+    if(bHasItemLinkIndex && QuickLinks[ItemLinkIndex].ValidValue < MinValidValue)
     {
         // If the current compressed byte was more than 7 bits,
         // set a quick-link item with pointer to tree item
@@ -783,14 +804,14 @@ unsigned int THuffmannTree::Compress(TOutputStream * os, void * pvInBuffer, int
     unsigned char * pbInBuffer = (unsigned char *)pvInBuffer;
     unsigned char * pbOutBuff = os->pbOutBuffer;
     unsigned char InputByte;
- 
+
     if(!BuildTree(CompressionType))
         return 0;
     bIsCmp0 = (CompressionType == 0);
 
     // Store the compression type into output buffer
     os->PutBits(CompressionType, 8);
- 
+
     // Process the entire input buffer
     while(pbInBuffer < pbInBufferEnd)
     {
@@ -802,10 +823,10 @@ unsigned int THuffmannTree::Compress(TOutputStream * os, void * pvInBuffer, int
         {
             // Encode the relationship
             EncodeOneByte(os, ItemsByByte[0x101]);
- 
+
             // Store the loaded byte into output stream
             os->PutBits(InputByte, 8);
- 
+
             if(!InsertNewBranchAndRebalance(pLast->DecompressedValue, InputByte))
                 return 0;
 
@@ -814,28 +835,28 @@ unsigned int THuffmannTree::Compress(TOutputStream * os, void * pvInBuffer, int
                 IncWeightsAndRebalance(ItemsByByte[InputByte]);
                 continue;
             }
-  
+
             IncWeightsAndRebalance(ItemsByByte[InputByte]);
         }
         else
         {
             EncodeOneByte(os, ItemsByByte[InputByte]);
         }
- 
+
         if(bIsCmp0)
         {
             IncWeightsAndRebalance(ItemsByByte[InputByte]);
         }
     }
- 
+
     // Put the termination mark to the compressed stream
     EncodeOneByte(os, ItemsByByte[0x100]);
- 
+
     // Flush the remaining bits
     os->Flush();
     return (unsigned int)(os->pbOutBuffer - pbOutBuff);
 }
- 
+
 // Decompression using Huffman tree (1500E450)
 unsigned int THuffmannTree::Decompress(void * pvOutBuffer, unsigned int cbOutLength, TInputStream * is)
 {
@@ -843,31 +864,33 @@ unsigned int THuffmannTree::Decompress(void * pvOutBuffer, unsigned int cbOutLen
     unsigned char * pbOutBuffer = (unsigned char *)pvOutBuffer;
     unsigned int DecompressedValue = 0;
     unsigned int CompressionType = 0;
-   
+
     // Test the output length. Must not be NULL.
     if(cbOutLength == 0)
         return 0;
- 
+
     // Get the compression type from the input stream
-    CompressionType = is->Get8Bits();
+    if(!is->Get8Bits(CompressionType))
+        return 0;
     bIsCmp0 = (CompressionType == 0) ? 1 : 0;
- 
+
     // Build the Huffman tree
     if(!BuildTree(CompressionType))
         return 0;
- 
+
     // Process the entire input buffer until end of the stream
     while((DecompressedValue = DecodeOneByte(is)) != 0x100)
     {
         // Did an error occur?
-        if(DecompressedValue == 0x1FF)          // An error occurred
+        if(DecompressedValue == HUFF_DECOMPRESS_ERROR)
             return 0;
 
         // Huffman tree needs to be modified
         if(DecompressedValue == 0x101)
         {
             // The decompressed byte is stored in the next 8 bits
-            DecompressedValue = is->Get8Bits();
+            if(!is->Get8Bits(DecompressedValue))
+                return 0;
 
             if(!InsertNewBranchAndRebalance(pLast->DecompressedValue, DecompressedValue))
                 return 0;
@@ -875,18 +898,18 @@ unsigned int THuffmannTree::Decompress(void * pvOutBuffer, unsigned int cbOutLen
             if(bIsCmp0 == 0)
                 IncWeightsAndRebalance(ItemsByByte[DecompressedValue]);
         }
- 
-        // A byte successfully decoded - store it in the output stream
-        *pbOutBuffer++ = (unsigned char)DecompressedValue;
+
+        // Store the byte to the output stream
         if(pbOutBuffer >= pbOutBufferEnd)
             break;
- 
+        *pbOutBuffer++ = (unsigned char)DecompressedValue;
+
         if(bIsCmp0)
         {
             IncWeightsAndRebalance(ItemsByByte[DecompressedValue]);
         }
     }
- 
+
     return (unsigned int)(pbOutBuffer - (unsigned char *)pvOutBuffer);
 }
 
