@@ -12,6 +12,7 @@
 
 #include <assert.h>
 #include <string.h>
+#include <limits.h>
 
 #include "pklib.h"
 
@@ -50,7 +51,7 @@ static void SortBuffer(TCmpStruct * pWork, unsigned char * buffer_begin, unsigne
 
     // Zero the entire "phash_to_index" table
     memset(pWork->phash_to_index, 0, sizeof(pWork->phash_to_index));
-    
+
     // Step 1: Count amount of each PAIR_HASH in the input buffer
     // The table will look like this:
     //  offs 0x000: Number of occurences of PAIR_HASH 0
@@ -60,7 +61,7 @@ static void SortBuffer(TCmpStruct * pWork, unsigned char * buffer_begin, unsigne
     for(buffer_ptr = buffer_begin; buffer_ptr < buffer_end; buffer_ptr++)
         pWork->phash_to_index[BYTE_PAIR_HASH(buffer_ptr)]++;
 
-    // Step 2: Convert the table to the array of PAIR_HASH amounts. 
+    // Step 2: Convert the table to the array of PAIR_HASH amounts.
     // Each element contains count of PAIR_HASHes that is less or equal
     // to element index
     // The table will look like this:
@@ -128,7 +129,7 @@ static void OutputBits(TCmpStruct * pWork, unsigned int nbits, unsigned long bit
     {
         pWork->out_bytes++;
         bit_buff >>= (8 - out_bits);
-        
+
         pWork->out_buff[pWork->out_bytes] = (unsigned char)bit_buff;
         pWork->out_bits &= 7;
     }
@@ -146,7 +147,7 @@ static void OutputBits(TCmpStruct * pWork, unsigned int nbits, unsigned long bit
 
 // This function searches for a repetition
 // (a previous occurence of the current byte sequence)
-// Returns length of the repetition, and stores the backward distance 
+// Returns length of the repetition, and stores the backward distance
 // to pWork structure.
 static unsigned int FindRep(TCmpStruct * pWork, unsigned char * input_data)
 {
@@ -187,7 +188,7 @@ static unsigned int FindRep(TCmpStruct * pWork, unsigned char * input_data)
     phash_offs = pWork->phash_offs + phash_offs_index;
     prev_repetition = pWork->work_buff + phash_offs[0];
     repetition_limit = input_data - 1;
-    
+
     // If the current PAIR_HASH was not encountered before,
     // we haven't found a repetition.
     if(prev_repetition >= repetition_limit)
@@ -213,7 +214,7 @@ static unsigned int FindRep(TCmpStruct * pWork, unsigned char * input_data)
             {
                 prev_repetition++;
                 input_data_ptr++;
-                
+
                 // Are the bytes different ?
                 if(*prev_repetition != *input_data_ptr)
                     break;
@@ -292,7 +293,7 @@ static unsigned int FindRep(TCmpStruct * pWork, unsigned char * input_data)
     //          The last repetition is the best one.
     //
 
-    pWork->offs09BC[0] = 0xFFFF;
+    pWork->offs09BC[0] = USHRT_MAX;
     pWork->offs09BC[1] = 0x0000;
     di_val = 0;
 
@@ -303,13 +304,13 @@ static unsigned int FindRep(TCmpStruct * pWork, unsigned char * input_data)
         if(input_data[offs_in_rep] != input_data[di_val])
         {
             di_val = pWork->offs09BC[di_val];
-            if(di_val != 0xFFFF)
+            if(di_val != USHRT_MAX)
                 continue;
         }
         pWork->offs09BC[++offs_in_rep] = ++di_val;
     }
 
-    // 
+    //
     // Now go through all the repetitions from the first found one
     // to the current input data, and check if any of them migh be
     // a start of a greater sequence match.
@@ -318,11 +319,11 @@ static unsigned int FindRep(TCmpStruct * pWork, unsigned char * input_data)
     prev_repetition = pWork->work_buff + phash_offs[0];
     prev_rep_end = prev_repetition + rep_length;
     rep_length2 = rep_length;
-    
+
     for(;;)
     {
         rep_length2 = pWork->offs09BC[rep_length2];
-        if(rep_length2 == 0xFFFF)
+        if(rep_length2 == USHRT_MAX)
             rep_length2 = 0;
 
         // Get the pointer to the previous repetition
@@ -395,7 +396,7 @@ static unsigned int FindRep(TCmpStruct * pWork, unsigned char * input_data)
                 if(input_data[offs_in_rep] != input_data[di_val])
                 {
                     di_val = pWork->offs09BC[di_val];
-                    if(di_val != 0xFFFF)
+                    if(di_val != USHRT_MAX)
                         continue;
                 }
                 pWork->offs09BC[++offs_in_rep] = ++di_val;
@@ -412,7 +413,7 @@ static void WriteCmpData(TCmpStruct * pWork)
     unsigned int save_rep_length;           // Saved length of current repetition
     unsigned int save_distance = 0;         // Saved distance of current repetition
     unsigned int rep_length;                // Length of the found repetition
-    unsigned int phase = 0;                 // 
+    unsigned int phase = 0;                 //
 
     // Store the compression type and dictionary size
     pWork->out_buff[0] = (char)pWork->ctype;
@@ -452,12 +453,12 @@ static void WriteCmpData(TCmpStruct * pWork)
         input_data_end = pWork->work_buff + pWork->dsize_bytes + total_loaded;
         if(input_data_ended)
             input_data_end += 0x204;
-        
+
         //
         // Warning: The end of the buffer passed to "SortBuffer" is actually 2 bytes beyond
         // valid data. It is questionable if this is actually a bug or not,
         // but it might cause the compressed data output to be dependent on random bytes
-        // that are in the buffer. 
+        // that are in the buffer.
         // To prevent that, the calling application must always zero the compression
         // buffer before passing it to "implode"
         //
@@ -466,7 +467,7 @@ static void WriteCmpData(TCmpStruct * pWork)
         // previously compressed data, if any.
         switch(phase)
         {
-            case 0: 
+            case 0:
                 SortBuffer(pWork, input_data, input_data_end + 1);
                 phase++;
                 if(pWork->dsize_bytes != 0x1000)
@@ -598,13 +599,11 @@ unsigned int PKEXPORT implode(
     unsigned int *dsize)
 {
     TCmpStruct * pWork = (TCmpStruct *)work_buf;
-    unsigned int nChCode;
     unsigned int nCount;
     unsigned int i;
     int nCount2;
 
     // Fill the work buffer information
-    // Note: The caller must zero the "work_buff" before passing it to implode
     pWork->read_buf    = read_buf;
     pWork->write_buf   = write_buf;
     pWork->dsize_bytes = *dsize;
@@ -637,11 +636,10 @@ unsigned int PKEXPORT implode(
     switch(*type)
     {
         case CMP_BINARY: // We will compress data with binary compression type
-            for(nChCode = 0, nCount = 0; nCount < 0x100; nCount++)
+            for(nCount = 0; nCount < 0x100; nCount++)
             {
                 pWork->nChBits[nCount]  = 9;
-                pWork->nChCodes[nCount] = (unsigned short)nChCode;
-                nChCode = (nChCode & 0x0000FFFF) + 2;
+                pWork->nChCodes[nCount] = nCount * 2;
             }
             break;
 
